<?php
namespace Bairwell\Autoloader\Tests;

class AutoloaderTest extends \PHPUnit_Framework_TestCase
{

    /**
     * @var \Bairwell\Autoloader
     */
    private $autoloader;

    public function setUp()
    {
        $this->autoloader = new \Bairwell\Autoloader();
        if (($this->autoloader instanceof \Bairwell\Autoloader) === FALSE) {
            $this->fail('Could not load autoloader');
        }
        spl_autoload_register(array($this->autoloader, 'load'));
    }

    public function tearDown()
    {
        spl_autoload_unregister(array($this->autoloader, 'load'));
    }

    public function testPrefixesEmpty()
    {
        $this->assertTrue(is_array($this->autoloader->getPrefixes()));
        $this->assertEmpty($this->autoloader->getPrefixes());
    }

    public function testAddPrefixFail()
    {
        $thisDir = 'FRED';
        $emess = NULL;
        try {
            $this->autoloader->addPrefix('Bairwell\Autoloader\Tests\Fixtures', $thisDir);
        } catch (\Exception $e) {
            $emess = $e->getMessage();
        }
        $this->assertEquals('FRED does not appear to be a directory', $emess);
    }

    public function testAddPrefixWorks()
    {
        $this->assertTrue(is_array($this->autoloader->getPrefixes()));
        $this->assertEmpty($this->autoloader->getPrefixes());
        $thisDir = __DIR__ . DIRECTORY_SEPARATOR . 'Fixtures';
        $this->autoloader->addPrefix('Bairwell\Autoloader\Tests\Fixtures', $thisDir);
        $prefixes = $this->autoloader->getPrefixes();
        $this->assertTrue(is_array($prefixes));
        $this->assertEquals(1, count($prefixes));
        $this->assertTrue(isset($prefixes['Bairwell\Autoloader\Tests\Fixtures']));
        $this->assertTrue(is_array($prefixes['Bairwell\Autoloader\Tests\Fixtures']));
        $this->assertEquals(1, count($prefixes['Bairwell\Autoloader\Tests\Fixtures']));
        $this->assertEquals($thisDir, $prefixes['Bairwell\Autoloader\Tests\Fixtures'][0]);
    }

    public function testAddMultiplePrefixWorks()
    {
        $this->assertTrue(is_array($this->autoloader->getPrefixes()));
        $this->assertEmpty($this->autoloader->getPrefixes());
        $thisDir = __DIR__ . DIRECTORY_SEPARATOR . 'Fixtures';
        $this->autoloader->addPrefix('Bairwell\Autoloader\Tests\Fixtures', $thisDir);
        $this->autoloader->addPrefix('Bairwell\Autoloader\Tests\Fixtures\Depth\Here', $thisDir);
        $this->autoloader->addPrefix('Bairwell\Autoloader\Tests\Fixtures\More', $thisDir);
        $this->autoloader->addPrefix('Awagoo\Awagoo\Fixtures\More', $thisDir);
        $this->autoloader->addPrefix('Awagoo\Awagoo\Fixtures\More', __DIR__);
        $prefixes = $this->autoloader->getPrefixes();
        $this->assertTrue(is_array($prefixes));
        $this->assertEquals(4, count($prefixes));
        $expected = Array(
            'Awagoo\Awagoo\Fixtures\More' => Array($thisDir, __DIR__),
            'Bairwell\Autoloader\Tests\Fixtures' => Array($thisDir),
            'Bairwell\Autoloader\Tests\Fixtures\More' => Array($thisDir),
            'Bairwell\Autoloader\Tests\Fixtures\Depth\Here' => Array($thisDir)

        );
        $diff = array_diff($expected, $prefixes);
        $this->assertEquals(0, count($diff));
    }

    public function testLoadAlreadyExists()
    {
        $emess = NULL;
        try {
            $this->autoloader->load('Exception');
        } catch (\Exception $e) {
            $emess = $e->getMessage();
        }
        $this->assertEquals('Class already loaded: Exception', $emess);
    }

    public function testLoadFileNotFound()
    {
        $emess = NULL;
        try {
            $this->autoloader->load('This\Is\A\Madeup\Class');
        } catch (\Exception $e) {
            $emess = $e->getMessage();
        }
        $this->assertEquals('File not found for This\Is\A\Madeup\Class', $emess);
    }

    public function testLoadFileKnownNamespace()
    {
        $thisDir = __DIR__ . DIRECTORY_SEPARATOR . 'Fixtures';
        $loaded = $this->autoloader->getLoaded();
        $this->assertTrue(is_array($loaded));
        $this->assertEquals(0, count($loaded));
        $this->autoloader->addPrefix('Bairwell\Autoloader\Tests\Fixtures', $thisDir);
        $this->assertFalse(class_exists('Bairwell\Autoloader\Tests\Fixtures\Tester', FALSE),
            'Should not already be loaded');
        $this->autoloader->load('Bairwell\Autoloader\Tests\Fixtures\Tester');
        $this->assertTrue(class_exists('\Bairwell\Autoloader\Tests\Fixtures\Tester', FALSE), 'Should now be loaded');
        $loaded = $this->autoloader->getLoaded();
        $this->assertTrue(is_array($loaded));
        $this->assertEquals(1, count($loaded));
    }

    public function testLoadFileKnownNamespaceUsingSPL()
    {
        $thisDir = __DIR__ . DIRECTORY_SEPARATOR . 'Fixtures';
        $this->autoloader->addPrefix('Bairwell\Autoloader\Tests\Fixtures', $thisDir);
        $this->assertFalse(class_exists('Bairwell\Autoloader\Tests\Fixtures\TestSPL', FALSE));
        \Bairwell\Autoloader\Tests\Fixtures\TestSPL::demo(123);
        $this->assertTrue(class_exists('Bairwell\Autoloader\Tests\Fixtures\TestSPL', FALSE));
    }

    public function testLoadFileDeepKnownNamespace()
    {
        $thisDir = __DIR__ . DIRECTORY_SEPARATOR . 'Fixtures';
        $this->autoloader->addPrefix('Bairwell\Autoloader\Tests\Fixtures', $thisDir);
        $this->autoloader->addPrefix('Random\Tests\Fixtures', $thisDir);
        $this->assertFalse(class_exists('Bairwell\Autoloader\Tests\Fixtures\Tester2', FALSE));
        $this->assertFalse(class_exists('Random\Tests\Fixtures\Deeper\Deep', FALSE));
        $this->autoloader->load('Random\Tests\Fixtures\Deeper\Deep');
        $this->autoloader->load('Bairwell\Autoloader\Tests\Fixtures\Tester2');
        $this->assertTrue(class_exists('Random\Tests\Fixtures\Deeper\Deep', FALSE));
        $this->assertTrue(class_exists('Bairwell\Autoloader\Tests\Fixtures\Tester2', FALSE));
    }

    public function testLoadFileBadClass()
    {
        $thisDir = __DIR__ . DIRECTORY_SEPARATOR . 'Fixtures';
        $this->autoloader->addPrefix('Bairwell\Autoloader\Tests\Fixtures', $thisDir);
        $emess = NULL;
        try {
            $this->autoloader->load('Bairwell\Autoloader\Tests\Fixtures\NotCorrect');
        } catch (\Exception $e) {
            $emess = $e->getMessage();
        }
        $this->assertEquals('Unable to load class Bairwell\Autoloader\Tests\Fixtures\NotCorrect', $emess);
    }

    public function testLoadNonNamespaced()
    {
        $thisDir = __DIR__ . DIRECTORY_SEPARATOR . 'Fixtures';
        $this->autoloader->addPrefix('Bairwell\Autoloader\Tests\Fixtures', $thisDir);
        $this->assertFalse(class_exists('PHPUnit_Framework_TestSuite_DataProvider', FALSE));
        /**
         * Try and load a PHP unit class we should not be using
         */
        $this->autoloader->load('PHPUnit_Framework_TestSuite_DataProvider');
        $this->assertTrue(class_exists('PHPUnit_Framework_TestSuite_DataProvider', FALSE));

    }
}